@@ -16,8 +16,6 @@
 
 """Tests for the Launchpad class."""
 
-__metaclass__ = type
-
 from contextlib import contextmanager
 import os
 import shutil
@@ -25,11 +23,7 @@ import socket
 import stat
 import tempfile
 import unittest
-
-try:
-    from unittest.mock import patch
-except ImportError:
-    from mock import patch
+from unittest.mock import patch
 import warnings
 
 from lazr.restfulclient.resource import ServiceRoot
@@ -351,11 +345,11 @@ class TestLaunchpadLoginWith(KeyringTest):
     """Tests for Launchpad.login_with()."""
 
     def setUp(self):
-        super(TestLaunchpadLoginWith, self).setUp()
+        super().setUp()
         self.temp_dir = tempfile.mkdtemp()
 
     def tearDown(self):
-        super(TestLaunchpadLoginWith, self).tearDown()
+        super().tearDown()
         shutil.rmtree(self.temp_dir)
 
     def test_dirs_created(self):
