@@ -29,9 +29,9 @@ from uaclient.entitlements.entitlement_status import (
     ContractStatus,
     UserFacingStatus,
 )
+from uaclient.files import machine_token
 from uaclient.files.state_files import status_cache_file
 from uaclient.types import MessagingOperationsDict, StaticAffordance
-from uaclient.util import is_config_value_true
 
 event = event_logger.get_event_logger()
 LOG = logging.getLogger(util.replace_top_level_logger_name(__name__))
@@ -54,12 +54,6 @@ class UAEntitlement(metaclass=abc.ABCMeta):
     # Optional URL for top-level product service information
     help_doc_url = None  # type: str
 
-    # Whether to assume yes to any messaging prompts
-    assume_yes = False
-
-    # Whether that entitlement is in beta stage
-    is_beta = False
-
     # Whether the entitlement supports the --access-only flag
     supports_access_only = False
 
@@ -116,7 +110,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         """The user-facing name shown for this entitlement"""
         if self.is_variant:
             return self.variant_name
-        elif self.cfg.machine_token_file.is_present:
+        elif self.machine_token_file.is_present:
             return (
                 self.entitlement_cfg.get("entitlement", {})
                 .get("affordances", {})
@@ -260,8 +254,6 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                 continue
             variant = variant_cls(
                 cfg=self.cfg,
-                assume_yes=self.assume_yes,
-                allow_beta=self.allow_beta,
                 called_name=self._called_name,
                 access_only=self.access_only,
             )
@@ -270,6 +262,27 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                 return variant
         return None
 
+    def variant_auto_select(self) -> bool:
+        """
+        Only implemented on variant classes.
+        Returns True if this variant should be auto-selected.
+        """
+        return False
+
+    @property
+    def default_variant(self):
+        """
+        Cannot actually set the return type because
+        https://github.com/python/typing/issues/266
+        affects xenial.
+
+        :rtype: Optional[Type["UAEntitlement"]]
+        """
+        variants = list(self.variants.values())
+        if variants:
+            return variants[0]
+        return None
+
     # Any custom messages to emit to the console or callables which are
     # handled at pre_enable, pre_disable, pre_install or post_enable stages
     @property
@@ -279,8 +292,6 @@ class UAEntitlement(metaclass=abc.ABCMeta):
     def __init__(
         self,
         cfg: Optional[config.UAConfig] = None,
-        assume_yes: bool = False,
-        allow_beta: bool = False,
         called_name: str = "",
         access_only: bool = False,
         purge: bool = False,
@@ -293,8 +304,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         if not cfg:
             cfg = config.UAConfig()
         self.cfg = cfg
-        self.assume_yes = assume_yes
-        self.allow_beta = allow_beta
+        self.machine_token_file = machine_token.get_machine_token_file()
         self.access_only = access_only
         self.purge = purge
         if extra_args is not None:
@@ -305,21 +315,9 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         self._valid_service = None  # type: Optional[bool]
         self._is_sources_list_updated = False
 
-    @property
-    def valid_service(self):
-        """Check if the service is marked as valid (non-beta)"""
-        if self._valid_service is None:
-            self._valid_service = (
-                not self.is_beta
-                or self.allow_beta
-                or is_config_value_true(self.cfg.cfg, "features.allow_beta")
-            )
-
-        return self._valid_service
-
     def _base_entitlement_cfg(self):
         return copy.deepcopy(
-            self.cfg.machine_token_file.entitlements.get(self.name, {})
+            self.machine_token_file.entitlements().get(self.name, {})
         )
 
     @property
@@ -369,7 +367,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             total_steps += 1
         for incompatible_service in self.blocking_incompatible_services():
             total_steps += incompatible_service.entitlement(
-                self.cfg
+                self.cfg,
             ).disable_steps()
         for required_service in self.blocking_required_services():
             total_steps += required_service.entitlement(
@@ -415,9 +413,6 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                 ),
             )
 
-        if not self.valid_service:
-            return (False, CanEnableFailure(CanEnableFailureReason.IS_BETA))
-
         applicability_status, details = self.applicability_status()
         if applicability_status == ApplicabilityStatus.INAPPLICABLE:
             return (
@@ -708,7 +703,9 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         """
         ret = []
         for service in self.incompatible_services:
-            ent_status, _ = service.entitlement(self.cfg).application_status()
+            ent_status, _ = service.entitlement(
+                cfg=self.cfg,
+            ).application_status()
             if ent_status in (
                 ApplicationStatus.ENABLED,
                 ApplicationStatus.WARNING,
@@ -750,7 +747,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             path_to_value="features.block_disable_on_enable",
         )
         for service in self.blocking_incompatible_services():
-            ent = service.entitlement(self.cfg, assume_yes=True)
+            ent = service.entitlement(self.cfg)
 
             e_msg = messages.E_INCOMPATIBLE_SERVICE_STOPS_ENABLE.format(
                 service_being_enabled=self.title,
@@ -809,7 +806,9 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         if the required service should be enabled before proceeding.
         """
         for required_service in self.blocking_required_services():
-            ent = required_service.entitlement(self.cfg, allow_beta=True)
+            ent = required_service.entitlement(
+                cfg=self.cfg,
+            )
             progress.emit(
                 "info",
                 messages.ENABLING_REQUIRED_SERVICE.format(service=ent.title),
@@ -985,7 +984,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         @param silent: Boolean set True to silence print/log of messages
         """
         for dependent_service_cls in self.blocking_dependent_services():
-            ent = dependent_service_cls(cfg=self.cfg, assume_yes=True)
+            ent = dependent_service_cls(cfg=self.cfg)
 
             progress.emit(
                 "info",
@@ -1243,6 +1242,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         orig_access: Dict[str, Any],
         deltas: Dict[str, Any],
         allow_enable: bool = False,
+        verbose: bool = True,
     ) -> bool:
         """Process any contract access deltas for this entitlement.
 
@@ -1253,6 +1253,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         :param allow_enable: Boolean set True if allowed to perform the enable
             operation. When False, a message will be logged to inform the user
             about the recommended enabled service.
+        :param verbose: If True, display output to stdout
 
         :return: True when delta operations are processed; False when noop.
         :raise: UbuntuProError when auto-enable fails unexpectedly.
@@ -1288,11 +1289,12 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                         self.name,
                     )
                     self.disable(api.ProgressWrapper())
-                    event.info(
-                        messages.DISABLE_DURING_CONTRACT_REFRESH.format(
-                            service=self.name
+                    if verbose:
+                        event.info(
+                            messages.DISABLE_DURING_CONTRACT_REFRESH.format(
+                                service=self.name
+                            )
                         )
-                    )
                 else:
                     fail_msg = fail.message_value if fail else ""
 
@@ -1302,11 +1304,12 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                         self.name,
                         fail_msg,
                     )
-                    event.info(
-                        messages.UNABLE_TO_DISABLE_DURING_CONTRACT_REFRESH.format(  # noqa: E501
-                            service=self.name
+                    if verbose:
+                        event.info(
+                            messages.UNABLE_TO_DISABLE_DURING_CONTRACT_REFRESH.format(  # noqa: E501
+                                service=self.name
+                            )
                         )
-                    )
             return True
 
         resourceToken = orig_access.get("resourceToken")
@@ -1317,22 +1320,23 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             delta_obligations, resourceToken
         )
 
-        if enable_by_default:
-            self.allow_beta = True
-
         can_enable, _ = self.can_enable()
         if can_enable and enable_by_default:
             if allow_enable:
                 msg = messages.ENABLE_BY_DEFAULT_TMPL.format(name=self.name)
 
-                event.info(msg, file_type=sys.stderr)
+                if verbose:
+                    event.info(msg, file_type=sys.stderr)
                 self.enable(api.ProgressWrapper())
-                event.info(messages.ENABLED_TMPL.format(title=self.title))
+
+                if verbose:
+                    event.info(messages.ENABLED_TMPL.format(title=self.title))
             else:
-                msg = messages.ENABLE_BY_DEFAULT_MANUAL_TMPL.format(
-                    name=self.name
-                )
-                event.info(msg, file_type=sys.stderr)
+                if verbose:
+                    msg = messages.ENABLE_BY_DEFAULT_MANUAL_TMPL.format(
+                        name=self.name
+                    )
+                    event.info(msg, file_type=sys.stderr)
             return True
 
         return False
